import datetime
import logging
import os
from pathlib import Path
import time

from .stream_reader import StreamReader
from h2cow.utils.configure import get_rtsp, read_config

def close_connection(readers: list):
    for reader in readers:
        reader.exit()

def get_frame(readers: list):
    for reader in readers: 
        reader.get_frame()

def get_streams(cfg: dict):
    readers = []
    for channel in cfg["channels"]:
        readers.append(
            StreamReader(
                channel, 
                get_rtsp(cfg, channel), 
                Path(
                    cfg["project_path"], 
                    cfg["image_recordings"]
                    )
                )
            )

def collection_loop(readers: list):
    try:
        print("Press crtl+c to exit.")
        while True:
            get_frame(readers)
            time.sleep(1)
    except KeyboardInterrupt:
        print("Exitting...")
        close_connection(readers)
        logging.info("Application closed")

def capture_frames(config: str):
    cfg = read_config(config)
    # logging.basicConfig(
    #     filename=Path(
    #         cfg["project_path"], 
    #         cfg["logs"], 
    #         f"data_collection{datetime.datetime.now().strftime('%Y%m%d%H%M%S')}.log"
    #         ), 
    #     level=logging.DEBUG, 
    #     filemode="w",
    #     format="%(asctime)s - %(levelname)s - %(message)s")
    readers = get_streams(cfg)
    offline_readers = [reader for reader in readers if not reader.isReady()]
    readers = [reader for reader in readers if reader.isReady()]
    close_connection(offline_readers)
    if len(readers) == 0:
        raise ConnectionError("Could not connect to any channel.")
    collection_loop(readers)
    


import logging

if __name__ == '__main__':
    logging_level = logging.DEBUG
    main_logger = logging.getLogger()
    main_logger.setLevel(logging_level)
    logging_handler = logging.FileHandler("log.log")
    logging_handler.setLevel(logging_level)
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    logging_handler.setFormatter(formatter)
    main_logger.addHandler(logging_handler)

    from create_project import test




import os
from pathlib import Path

from h2cow.utils import configure


def new(project_directory: str, server_ip: str):
    os.makedirs(project_directory)
    os.makedirs(os.path.join(project_directory, "recordings"))

    cfg = configure.create_template_config()

    # project directory
    cfg["project_path"] = project_directory
    
    # rtsp url
    cfg["server_ip"] = server_ip
    cfg["port"] = "554"
    cfg["user_name"] = "admin"
    cfg["password"] = "admin"
    cfg["channels"] = list(range(1,7))

    # directories
    cfg["image_recordings"] = str(Path("recordings", "images"))
    cfg["logs"] = "logs"
    cfg["water_recordings"] = str(Path("recordings", "water_gauge"))

    configure.write_config(cfg, Path(project_directory, "config.yaml"))
    return cfg


#from dlclive import DLCLive, Processor
import os
import deeplabcut
#dlc_proc = Processor()

output_path = deeplabcut.DownSampleVideo(os.path.join("C:/Users/danie/Desktop/packaging_tutorial/Data", "mijnkoe.mp4"), width=300)
print(output_path)


import datetime
import logging
import os
from pathlib import Path
import threading
import time

import cv2 

from h2cow.utils import configure

class StreamReader:
    def __init__(self, channel: int, rtsp_url: str, save_path: str):
        self.Channel = channel
        self.__save_path = save_path
        self.__stream = self.__open_stream(rtsp_url)
        self.__ready = False
        self.__status = False
        self.__frame = None
        self.__online = True
        if not self.__stream is None:
            self.__thread = threading.Thread(target=self.__update)
            self.__thread.start()
        waited = 0
        while not self.__ready and waited < 10:
            waited += 0.1
            time.sleep(0.1)

    def exit(self):
        self.__online = False
        self.__thread.join()
        logging.info(f"Closed stream on channel {self.Channel}.")

    def get_frame(self):
        path = Path(
            self.__save_path, 
            f"channel_{self.Channel}", 
            f"{datetime.datetime.now().strftime('%Y%m%d')}", 
            f"{datetime.datetime.now().strftime('%H')}"
            )
        if not os.path.isdir(path):
            os.makedirs(path)
        cv2.imwrite(
            Path(
                path, 
                f"{datetime.datetime.now().strftime('%M%S')}.jpeg"
                ),
            self.__frame
            )

    def isReady(self):
        return self.__status

    def __open_stream(self, rtsp_url: str):
        logging.debug(f"Starting to open stream on channel {self.Channel}")
        cap = cv2.VideoCapture(rtsp_url, cv2.CAP_FFMPEG)
        if not cap.isOpened():
            logging.error(f"Unable to open stream on channel {self.Channel}.")
            return None
        logging.info(f"Opened stream on channel {self.Channel}.")
        return cap

    def __update(self):
        while self.__online:
            if self.__stream.isOpened():
                self.__status, self.__frame = self.__stream.read()
                if self.__status and not self.__ready:
                    self.__ready = True
                    logging.info(f"Stream on channel {self.Channel} ready.")


import cv2
import numpy as np
import time

def find_detections(image, outputs, threshold=.85):
    bbox = []
    hT, wT, cT = image.shape
    for output in outputs:
        for box in output:
            # coco label 19 is cow, so we need index 24
            # because first five are bbox parameters
            score = box[24]
            if score >= threshold:
                w,h = int(box[2]*wT), int(box[3]*hT)
                x,y = int(box[0]*wT - w/2), int(box[1]*hT - h/2)
                bbox.append([x,y,w,h])
    return bbox

VIDEO_PATH = "Koeien - 44022.mp4"
IMAGE_PATH = "images/"
EXTENSION = ".jpg"
FREQUENCY = 1   # how often to take a snapshot in seconds
width, height = 320, 320

yolo = cv2.dnn.readNetFromDarknet("yolov3-spp.cfg", "yolov3-spp.weights")
yolo.setPreferableBackend(cv2.dnn.DNN_BACKEND_OPENCV)
yolo.setPreferableTarget(cv2.dnn.DNN_TARGET_CPU)


cap = cv2.VideoCapture(VIDEO_PATH)
while True: 
    #frame = cv2.imread("images/Koeien - 44022_0.jpg")
    ret, frame = cap.read()
    if not ret: break
    blob = cv2.dnn.blobFromImage(frame, 1/255, (width,height), [0,0,0], 1, crop=False)
    yolo.setInput(blob)

    layerNames = yolo.getLayerNames()
    outputNames = [layerNames[(i-1)] for i in yolo.getUnconnectedOutLayers()]
    
    outputs = yolo.forward(outputNames)
    detections = find_detections(frame, outputs)
    for box in detections:
        x,y,w,h = box[0],box[1],box[2],box[3]
        cv2.rectangle(frame, (x, y), (x+w, y+h), (255,0,255), 2)
    cv2.imshow("Image", frame)
    key = cv2.waitKey(20)
    if key == 27: # exit on ESC
        break
cap.release()


from ruamel.yaml import YAML

def write_config(config_dict, yaml_path):
    with open(yaml_path, "w") as f:
        yaml = YAML()
        yaml.dump(config_dict, f)

def read_config(yaml_path):
    with open(yaml_path, "r") as f:
        yaml = YAML()
        return yaml.load(f)

def get_rtsp(cfg, channel):
    return "rtsp://"\
        f"{cfg['server_ip']}"\
        ":"\
        f"{cfg['port']}"\
        "/user="\
        f"{cfg['user_name']}"\
        "&password="\
        f"{cfg['password']}"\
        "&channel="\
        f"{channel}"\
        "&stream=0"
    
def create_template_config():
    """
    This creates the template for config.yaml.
    """
    yaml_str = '''
# Project path (edit when moving around)
    project_path:

# RTSP url (edit where needed)
    server_ip:
    port: 
    user_name:
    password:
    channels:

# Default directories (do not edit)
    image_recordings:
    logs:
    water_recordings:
    '''
    yaml = YAML()
    cfg = yaml.load(yaml_str)
    return cfg



import os
from pathlib import Path

import click

from h2cow.create_project.new_project import new
from h2cow.data_collection.data_collection import capture_frames

@click.group()
def main():
    click.echo("h2cow started.")

###########################################################################################################################

@click.option("-p", "--path", prompt="Project path", help="Where to create project.")
@click.option("-a", "--address", prompt="Server IP-address", help="IP-address of rtsp server.")
@main.command()
def create_project(path, address):
    """Creates a new default project for h2cow."""
    path = os.path.abspath(path)
    try:
        proj = new(path, address)
    except FileExistsError:
        click.echo(f"Project already exists at '{path}': Exitting.")
        exit()
    click.echo(f"Created project at '{path}'. Please edit config.yaml there before continuing.")



###########################################################################################################################

@main.command()
@click.argument("config")
def capture(config):
    """Starting capturing data from ip cameras."""
    if not os.path.isdir(config):
        click.echo(f"Invalid configuration file '{Path(config, 'config.yaml')}'.\n Have you created a project yet?")
        exit()
    capture_frames(Path(config, "config.yaml"))
    


###########################################################################################################################


import setuptools
from setuptools import setup

version = {}
with open("h2cow/_version.py") as fp:
    exec(fp.read(), version)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    version=version["__version__"],
    name="h2cow",
    license = "MIT",
    description="Monitoring drinking behaviour of animals.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Daniël M. van Herwijnen",
    author_email="danielvh8@gmail.com",
    url = "https://github.com/danielvh8/Master-Research-Project-2022",
    packages=setuptools.find_packages(),
    install_requires=[
        "click==8.1.3",
        "deeplabcut==2.2.2",
        "numpy>=1.18.5",
        "opencv_contrib_python==4.6.0.66",
        "ruamel.yaml>=0.15.0",
        "setuptools<60"
    ],
    entry_points={
        'console_scripts':{
            'h2cow = h2cow.cli:main'
        }
    },)

    



import datetime
import random
import time

# import RPi.GPIO as GPIO

GPIO_signal = True

def get_signal():
    global GPIO_signal
    r_num = random.random()
    if r_num > .7:
        return not GPIO_signal
    return GPIO_signal

def record():
    with open("water_recordings.txt", "a") as file:
        file.write(f"{datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S:%f')}\n")

if __name__ == "__main__":
    f = open("water_recordings.txt", "w")
    f.close()
    try:
        while True:
            if GPIO_signal != get_signal():
                GPIO_signal = not GPIO_signal
                record()
            time.sleep(1)
    except KeyboardInterrupt:
        exit()